# Modules hell =)

AC_DEFUN([PS_MODULES],
[
	AC_MSG_CHECKING([which modules must be compiled])
	
	MODULES_LIST=
	
	EPO_ORIGIN_PATH=`pwd`
	cd "$srcdir/src/modules"
	
	for MODULE in * ; do
		if test ! -d "$MODULE" || test "$MODULE" = '.' || test "$MODULE" = '..' ; then
			continue
		fi
		
		if test -f "$MODULE/configure" || test -f "$MODULE/Makefile.in" || test -f "$MODULE/Makefile" ; then
			MODULES_LIST="$MODULES_LIST$MODULE "
		fi
	done
	
	cd "$EPO_ORIGIN_PATH"
	
	AC_MSG_RESULT($MODULES_LIST)
	
	MODULES_LIBS=
	MODULES_DEPS=
	
	for MODULE in $MODULES_LIST ; do
		MODULES_LIBS="$MODULES_LIBS-dlopen \$(top_builddir)/src/modules/$MODULE/$MODULE.la "
		MODULES_DEPS="$MODULES_DEPS\$(top_builddir)/src/modules/$MODULE/$MODULE.la "
		
		if test -f $srcdir/src/modules/$MODULE/configure ; then
	    	AC_CONFIG_SUBDIRS([src/modules/$MODULE])
		fi
	done
	
	AC_SUBST(MODULES_LIST)
	AC_SUBST(MODULES_LIBS)
	AC_SUBST(MODULES_DEPS)
]) dnl AC_DEFUN

AC_DEFUN([PS_ARG_WITH_IRCD],
[AC_MSG_CHECKING(for IRCd)
AC_ARG_WITH(ircd,
[AS_HELP_STRING([[--with-ircd=type]], [select IRCd type (dreamforge, bahamut, unreal, ultimate, sequana, kewl)])],
[case $withval in
	d*|D*) AC_DEFINE(IRC_DREAMFORGE, [], [Define if Dreamforge or derivated is used])
	       ircdtype="Dreamforge"
	       ;;
	b*|B*) AC_DEFINE(IRC_BAHAMUT, [], [Define if Bahamut or derivated is used])
	       ircdtype="Bahamut"
	       ;;
    un*|uN*|Un*|UN*) AC_DEFINE(IRC_DREAMFORGE, [])
           AC_DEFINE(IRC_UNREAL, [], [Define if Unreal or derivated is used])
	       ircdtype="Unreal"
	       ;;
	ul*|uL*|Ul*|UL*) AC_DEFINE(IRC_DREAMFORGE, [])
           AC_DEFINE(IRC_ULTIMATE, [], [Define if Ultimate or derivated is used])
	       ircdtype="Ultimate"
	       ;;
	s*|S*) AC_DEFINE(IRC_BAHAMUT, [])
	       AC_DEFINE(IRC_SEQUANA, , [Define if Sequana or derivated is used])
	       ircdtype="Sequana"
	       ;;
    l*|L*) AC_DEFINE(IRC_BAHAMUT, [])
	       AC_DEFINE(IRC_SEQUANA_LIGHT, , [Define if Sequana Light or derivated is used])
	       ircdtype="Sequana Light"
	       ;;
	k*|K*) AC_DEFINE(IRC_BAHAMUT, [])
	       AC_DEFINE(IRC_KEWL, , [Define if Kewl or derivated is used])
	       ircdtype="Kewl"
	       ;;
	    *) AC_MSG_ERROR(invalid IRCd type)
	       ;; 
esac
AC_MSG_RESULT($ircdtype)],
[AC_MSG_ERROR([no --with-ircd option specified; see the INSTALL file])]) dnl AC_ARG_WITH
]) dnl AC_DEFUN

AC_DEFUN([PS_ARG_WITH_BZLIB],
[
	BZLIB_ROOT="/usr/local"
	if test ! -f "${BZLIB_ROOT}/include/bzlib.h" ; then
		BZLIB_ROOT="/usr"
	fi

	AC_ARG_WITH(bzlib,
	[AS_HELP_STRING([[--with-bzlib=DIR]], [  alternative root of the bzlib installation (defaults to /usr/local or /usr if not found in /usr/local)])],
	[if test "$withval" != "no" ; then
		if test -d "$withval" ; then
			BZLIB_ROOT="$withval"
		else
			AC_MSG_WARN([$withval does not exist, trying generic location for bzlib library])
		fi
	fi]) dnl AC_ARG_WITH
	
	BZLIB_OLD_CPPFLAGS="$CPPFLAGS"
	CPPFLAGS="$CPPFLAGS -I${BZLIB_ROOT}/include"
	BZLIB_OLD_LDFLAGS="$LDFLAGS"
	LDFLAGS="$LDFLAGS -L${BZLIB_ROOT}/lib"
	
	AC_CHECK_LIB(bz2, BZ2_bzCompress, [bzlib_cv_libbz2="yes"], [bzlib_cv_libbz2="no"])
	AC_CHECK_HEADER(bzlib.h, [bzlib_cv_bzlib_h="yes"], [bzlib_cv_bzlib_h="no"])
	
	CPPFLAGS="$BZLIB_OLD_CPPFLAGS"
	LDFLAGS="$BZLIB_OLD_LDFLAGS"
	
	if test "$bzlib_cv_libbz2" = "yes" && test "$bzlib_cv_bzlib_h" = "yes" ; then
		BZLIB_CPPFLAGS="-I${BZLIB_ROOT}/include"
		BZLIB_LDFLAGS="-L${BZLIB_ROOT}/lib"
		BZLIB_LIBS="-lbz2"
		AC_DEFINE(HAVE_BZLIB, 1, [Define if you have bzlib])
		bzlib_cv_with_bzlib="yes"
	else
		BZLIB_CPPFLAGS=""
		BZLIB_LDFLAGS=""
		BZLIB_LIBS=""
		bzlib_cv_with_bzlib="no"
	fi
	
	AC_SUBST(BZLIB_CPPFLAGS)
	AC_SUBST(BZLIB_LDFLAGS)
	AC_SUBST(BZLIB_LIBS)
	
	AM_CONDITIONAL(WITH_BZLIB, test "$bzlib_cv_with_bzlib" = "yes")
]) dnl AC_DEFUN

AC_DEFUN([PS_ARG_WITH_MYSQL],
[
	MYSQL_ROOT=""
	
	AC_ARG_WITH(mysql,
	[AS_HELP_STRING([[--with-mysql=DIR]], [  alternative for the mysql_config utility])],
	[if test "$withval" != "no" ; then
		if test -d "$withval" ; then
			MYSQL_ROOT="$withval"
		else
			AC_MSG_WARN([$withval does not exist, trying generic location for mysql_config])
		fi
	fi]) dnl AC_ARG_WITH

	if test -z "$MYSQL_ROOT" ; then
		AC_PATH_PROG(MYSQL_CONFIG, mysql_config, [])
	else
		AC_PATH_PROG(MYSQL_CONFIG, mysql_config, [], "$MYSQL_ROOT:$PATH")
	fi
	
	MYSQL_FOUND="no"
	
	if test -n "$MYSQL_CONFIG"
	then
		AC_MSG_CHECKING(for mysql version >= 4.1.13)
		MYSQL_VERSION=`"$MYSQL_CONFIG" --version`
		MYSQL_VERMAJ=`echo -n "$MYSQL_VERSION" |cut -d '.' -f 1`
		MYSQL_VERMIN=`echo -n "$MYSQL_VERSION" |cut -d '.' -f 2`
		MYSQL_VERREL=`echo -n "$MYSQL_VERSION" |cut -d '.' -f 3`
		MYSQL_VERREL=`echo -n "$MYSQL_VERREL" |cut -b 1-2 |tr -d "[:alpha:][:cntrl:][:punct:][:space:]"`
		
		if test "$MYSQL_VERMAJ" -lt 4 
		then
			MYSQL_VERGOOD=no
		else
			if test "$MYSQL_VERMAJ" -gt 4 
			then
				if test "$MYSQL_VERMAJ" = 5 && test "$MYSQL_VERMIN" = 0 &&
						test "$MYSQL_VERREL" -gt 3 && test "$MYSQL_VERREL" -lt 13
				then
					MYSQL_VERGOOD=no
				else
					MYSQL_VERGOOD=yes
				fi
			else
				# MYSQL_VERMAJ == 4
				if test "$MYSQL_VERMIN" -lt 1 
				then
					MYSQL_VERGOOD=no
				else
					# MYSQL_VERMIN == 1
					if test "$MYSQL_VERREL" -lt 13 
					then
						MYSQL_VERGOOD=no
					else
						MYSQL_VERGOOD=yes
					fi
				fi
			fi
		fi
		
		AC_MSG_RESULT([$MYSQL_VERGOOD ($MYSQL_VERSION)])
		
		if test "$MYSQL_VERGOOD" = "yes"
		then
			AC_MSG_CHECKING(for mysql compiler flags)
			MYSQL_CFLAGS=`"$MYSQL_CONFIG" --cflags`
			MYSQL_CFLAGS=`echo "$MYSQL_CFLAGS" |sed '
					s/-march[a-zA-Z0-9=-]* //g' |sed '
					s/-mcpu[a-zA-Z0-9=-]* //g' |sed 's/-fomit-frame-pointer//g'`
			AC_MSG_RESULT($MYSQL_CFLAGS)
			AC_MSG_CHECKING(for mysql libraries)
			MYSQL_LIBS=`"$MYSQL_CONFIG" --libs`
			AC_MSG_RESULT($MYSQL_LIBS)
			
			AC_DEFINE(HAVE_MYSQL, 1, [Define if you have mysql])
			MYSQL_FOUND="yes"
		fi
	fi
	
	if test "$MYSQL_FOUND" = "no"
	then
		MYSQL_CFLAGS=""
		MYSQL_LIBS=""
	fi
	
	AC_SUBST(MYSQL_CFLAGS)
	AC_SUBST(MYSQL_LIBS)
	
	AM_CONDITIONAL(WITH_MYSQL, test "$MYSQL_FOUND" = "yes")
]) dnl AC_DEFUN

AC_DEFUN([PS_ARG_WITH_OPENSSL],
[
	OPENSSL_ROOT="/usr/local"
	if test ! -f "${OPENSSL_ROOT}/include/openssl/crypto.h" ; then
		OPENSSL_ROOT="/usr"
	fi

	AC_ARG_WITH(openssl,
	[AS_HELP_STRING([[--with-openssl=DIR]], [  alternative root of the openssl installation (defaults to /usr/local or /usr if not found in /usr/local)])],
	[if test "$withval" != "no" ; then
		if test -d "$withval" ; then
			OPENSSL_ROOT="$withval"
		else
			AC_MSG_WARN([$withval does not exist, trying generic location for openssl library])
		fi
	fi]) dnl AC_ARG_WITH
	
	OPENSSL_OLD_CPPFLAGS="$CPPFLAGS"
	CPPFLAGS="$CPPFLAGS -I${OPENSSL_ROOT}/include"
	OPENSSL_OLD_LDFLAGS="$LDFLAGS"
	LDFLAGS="$LDFLAGS -L${OPENSSL_ROOT}/lib"
	
	AC_CHECK_LIB(crypto, EVP_DigestInit_ex, [openssl_cv_libcrypto="yes"], [openssl_cv_libcrypto="no"])
	AC_CHECK_HEADERS(openssl/evp.h openssl/rand.h, [openssl_cv_headers="yes"], [openssl_cv_headers="no"])
	
	CPPFLAGS="$OPENSSL_OLD_CPPFLAGS"
	LDFLAGS="$OPENSSL_OLD_LDFLAGS"
	
	if test "$openssl_cv_libcrypto" = "yes" && test "$openssl_cv_headers" = "yes" ; then
		OPENSSL_CPPFLAGS="-I${OPENSSL_ROOT}/include"
		OPENSSL_LDFLAGS="-L${OPENSSL_ROOT}/lib"
		OPENSSL_LIBS="-lcrypto"
		AC_DEFINE(HAVE_OPENSSL, 1, [Define if you have openssl])
		openssl_cv_with_openssl="yes"
	else
		OPENSSL_CPPFLAGS=""
		OPENSSL_LDFLAGS=""
		OPENSSL_LIBS=""
		openssl_cv_with_openssl="no"
	fi
	
	AC_SUBST(OPENSSL_CPPFLAGS)
	AC_SUBST(OPENSSL_LDFLAGS)
	AC_SUBST(OPENSSL_LIBS)
	
	AM_CONDITIONAL(WITH_OPENSSL, test "$openssl_cv_with_openssl" = "yes")
]) dnl AC_DEFUN

AC_DEFUN([PS_ARG_WITH_ZLIB],
[
	ZLIB_ROOT="/usr/local"
	if test ! -f "${ZLIB_ROOT}/include/zlib.h" ; then
		ZLIB_ROOT="/usr"
	fi

	AC_ARG_WITH(zlib,
	[AS_HELP_STRING([[--with-zlib=DIR]], [  alternative root of the zlib installation (defaults to /usr/local or /usr if not found in /usr/local)])],
	[if test "$withval" != "no" ; then
		if test -d "$withval" ; then
			ZLIB_ROOT="$withval"
		else
			AC_MSG_WARN([$withval does not exist, trying generic location for zlib library])
		fi
	fi]) dnl AC_ARG_WITH
	
	ZLIB_OLD_CPPFLAGS="$CPPFLAGS"
	CPPFLAGS="$CPPFLAGS -I${ZLIB_ROOT}/include"
	ZLIB_OLD_LDFLAGS="$LDFLAGS"
	LDFLAGS="$LDFLAGS -L${ZLIB_ROOT}/lib"
	
	AC_CHECK_LIB(z, inflateEnd, [zlib_cv_libz="yes"], [zlib_cv_libz="no"])
	AC_CHECK_HEADER(zlib.h, [zlib_cv_zlib_h="yes"], [zlib_cv_zlib_h="no"])
	
	CPPFLAGS="$ZLIB_OLD_CPPFLAGS"
	LDFLAGS="$ZLIB_OLD_LDFLAGS"
	
	if test "$zlib_cv_libz" = "yes" && test "$zlib_cv_zlib_h" = "yes" ; then
		ZLIB_CPPFLAGS="-I${ZLIB_ROOT}/include"
		ZLIB_LDFLAGS="-L${ZLIB_ROOT}/lib"
		ZLIB_LIBS="-lz"
		AC_DEFINE(HAVE_ZLIB, 1, [Define if you have zlib])
		zlib_cv_with_zlib="yes"
	else
		ZLIB_CPPFLAGS=""
		ZLIB_LDFLAGS=""
		ZLIB_LIBS=""
		zlib_cv_with_zlib="no"
	fi
	
	AC_SUBST(ZLIB_CPPFLAGS)
	AC_SUBST(ZLIB_LDFLAGS)
	AC_SUBST(ZLIB_LIBS)
	
	AM_CONDITIONAL(WITH_ZLIB, test "$zlib_cv_with_zlib" = "yes")
]) dnl AC_DEFUN

# Tests for the poll() function

AC_DEFUN([PS_FUNC_POLL],
[
	AC_CHECK_FUNCS(poll)
	AM_CONDITIONAL(WITH_POLL, test "$ac_cv_func_poll" = "yes")
]) dnl PS_FUNC_POLL

# Tests for an ISO C99 snprintf that doesn't overflow its buffer.

AC_DEFUN([EPO_FUNC_SNPRINTF],
[
	AC_CACHE_CHECK([for a working ISO C99 compliant snprintf], epo_cv_func_snprintf,
	[
		AC_TRY_RUN([#include <stdio.h>
#include <stdlib.h>

			int main() 
			{
				char buf[16];
				
				buf[4] = 'E';
				
				/* ISO C99 compliant snprintf always return the number of
				 * bytes that should have been produced. */
				if (snprintf(buf, 4, "%s", "zyxwvut") != 7)
					exit(1);
				
				/* Some old snprintf implementation ignore the length and
				 * overrun the buffer. */
				if (buf[4] != 'E')
					exit(1);
				
				/* ISO C99 compliant snprintf allow length to be 0 while
				 * still computing a valid return value. */
				if (snprintf(buf, 0, "%s", "zyxwvut") != 7)
					exit(1);
				
				exit(0);
			}
		], [epo_cv_func_snprintf="yes"], [epo_cv_func_snprintf="no"])
	])
	
	if test "$epo_cv_func_snprintf" = "no" ; then
		AC_SEARCH_LIBS([tan], [m])
		AC_CHECK_HEADER([math.h])
		AC_LIBOBJ(snprintf)
	else
		AC_DEFINE(HAVE_SNPRINTF, 1, [Define to 1 if you have a working ISO C99 compliant `snprintf' function.])
	fi
])

# Tests for an ISO C99 vsnprintf that doesn't overflow its buffer.

AC_DEFUN([EPO_FUNC_VSNPRINTF],
[
	AC_CACHE_CHECK([for a working ISO C99 compliant vsnprintf], epo_cv_func_vsnprintf,
	[
		AC_TRY_RUN([#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>

			int test_it(char *format, ...) 
			{
				char buf[16];
				va_list ap;
				
				buf[4] = 'E';
				
				va_start(ap, format);
				
				/* ISO C99 compliant snprintf always return the number of
				 * bytes that should have been produced. */
				if (vsnprintf(buf, 4, format, ap) != 7)
					return 1;
				
				va_end(ap);
				
				/* Some old snprintf implementation ignore the length and
				 * overrun the buffer. */
				if (buf[4] != 'E')
					return 1;
				
				va_start(ap, format);
				
				/* ISO C99 compliant snprintf allow length to be 0 while
				 * still computing a valid return value. */
				if (vsnprintf(buf, 0, format, ap) != 7)
					return 1;
				
				va_end(ap);
				
				return 0;
			}

			int main() 
			{
				exit(test_it("%s", "zyxwvut"));
			}
		], [epo_cv_func_vsnprintf="yes"], [epo_cv_func_vsnprintf="no"])
	])
	
	if test "$epo_cv_func_vsnprintf" = "no" && test "$epo_cv_func_snprintf" = "yes" ; then
		AC_SEARCH_LIBS([tan], [m])
		AC_CHECK_HEADER([math.h])
		AC_LIBOBJ(snprintf)
	elif test "$epo_cv_func_vsnprintf" = "yes" ; then
		AC_DEFINE(HAVE_VSNPRINTF, 1, [Define to 1 if you have a working ISO C99 compliant `vsnprintf' function.])
	fi
])