/* Data fields.
 *
 * PegSoft databases library (c) 2005 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file COPYING); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <system/headers.h>

#include "databases/field.h"

using namespace std;

/*************************************************************************/
	
namespace databases
{

/*************************************************************************/

uint16_t Field::attrs_add(uint16_t attrs)
{
	if (m_setup)
		return m_attrs;
	
	return (m_attrs |= attrs);
}

/*************************************************************************/

uint16_t Field::attrs_remove(uint16_t attrs)
{
	if (m_setup)
		return m_attrs;
	
	return (m_attrs &= ~attrs);
}

/*************************************************************************/

char const *Field::default_value(char const *defval)
{
	if (m_setup)
		return m_default;
	
	/* "" == NULL =) */
	if (defval && !*defval)
		defval = NULL;
	
	return (m_default = defval);
}

/*************************************************************************/

char const *Field::name(char const *name)
{
	if (m_setup)
		return m_name;
	
	return (m_name = name);
}

/*************************************************************************/

bool Field::setup()
{
	if (!m_name || !m_minver || (m_type < 1) || (m_type > ftCount))
		return false;
	
	char *end;
	
	switch (m_type)
	{
		case ftString:
			if (!m_size || (m_attrs & (faOwner | faOwnerUnique)))
				return false;
			
			break;
		case ftInteger:
			if (m_size < 1 || m_size > 4 || m_attrs || !m_default)
				return false;
			
			m_idefault = strtol(m_default, &end, 10);
			if (*end)
				return false;
			break;
		case ftUnsigned:
			if (m_size < 1 || m_size > 4 || m_attrs || !m_default)
				return false;
			
			m_idefault = strtoul(m_default, &end, 10);
			if (*end)
				return false;
			break;
		case ftUnixTime:
			if (m_attrs || !m_default)
				return false;
			
			m_utdefault = strtoul(m_default, &end, 10);
			if (*end)
				return false;
			break;
		case ftIdentifier:
			if ((m_attrs & faCase) || !m_default)
				return false;
		
			m_ridefault = Record::strtori(m_default, &end, 10);
			if (*end)
				return false;
			break;
		default:
			break;
	}
	
	return (m_setup = true);
}

/*************************************************************************/

size_t Field::size(size_t size)
{
	if (m_setup)
		return m_size;
	
	return (m_size = size);
}


/*************************************************************************/

uint16_t Field::tag(uint16_t tag)
{
	if (m_setup)
		return m_tag;
	
	return (m_tag = tag);
}

/*************************************************************************/

uint16_t Field::type(uint16_t type)
{
	if (m_setup)
		return m_type;
	
	return (m_type = type);
}


/*************************************************************************/

uint16_t Field::version(int type) const
{
	switch (type)
	{
		case Database::vtMinimum:
			return m_minver;
		case Database::vtMaximum:
			return m_maxver;
		default:
			return 0;
	}
}

/*************************************************************************/

uint16_t Field::version(int type, uint16_t ver)
{
	if (m_setup)
		return version(type);
	
	switch (type)
	{
		case Database::vtMinimum:
			return (m_minver = ver);
		case Database::vtMaximum:
			return (m_maxver = ver);
		default:
			return (ver ? 0 : 1);
	};
}

/*************************************************************************/

} /* namespace databases */

/*************************************************************************/
