/* Log configuration directives.
 *
 * PegSoft log subsystem library (c) 2005 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software (the COPYING file); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 */

#include <system/headers.h>

#include "log/conf.h"
#include "log/levels.h"

using namespace std;
using namespace conf;

/*************************************************************************/

namespace xlog
{

/*************************************************************************/

LogRules &LogRules::operator=(LogRules const &right)
{
	if (&right != this)
	{
		ConfRules::operator=(right);
		
		m_accept = right.m_accept;
		m_reject = right.m_reject;
	}
	
	return *this;
}

/*************************************************************************/

#define CHECK_ADD(x, y, z) if (!(x).add((y))) { \
	(z).errorf("Can't add directive %s to %s: out of memory?", \
			(y).name().c_str(), m_name.c_str()); \
	return false; }

bool LogRules::add(Conf &conf, Conf &new_conf)
{
	m_accept.container(m_rules);
	CHECK_ADD(new_conf, m_accept, conf);
	m_reject.container(m_rules);
	CHECK_ADD(new_conf, m_reject, conf);
	
	return true;
}

#undef CHECK_ADD

/*************************************************************************/

ConfRules::Rule const *LogRules::match(LogLevel const &level) const
{
	container_type::const_iterator it, end;
	
	for (it = m_rules.begin(), end = m_rules.end(); it != end; ++it)
	{
		if (level == it->value())
			return &(*it);
	}
	
	return 0;
}

/*************************************************************************/

} /* namespace xlog */

/*************************************************************************/
