/* Base64 encoding.
 *
 * PegSoft miscellaneous library (c) 2004 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software (the COPYING file); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 */

#include <system/headers.h>

#include "misc/misc.h"

using namespace std;

/*************************************************************************/

namespace misc
{

/*************************************************************************/
/*************************************************************************/

/* The following code has been written by Bob Trower
 * (http://base64.sourceforge.net/) with a few modifications. 
 */

/*
LICENCE:        Copyright (c) 2001 Bob Trower, Trantor Standard Systems Inc.

                Permission is hereby granted, free of charge, to any person
                obtaining a copy of this software and associated
                documentation files (the "Software"), to deal in the
                Software without restriction, including without limitation
                the rights to use, copy, modify, merge, publish, distribute,
                sublicense, and/or sell copies of the Software, and to
                permit persons to whom the Software is furnished to do so,
                subject to the following conditions:

                The above copyright notice and this permission notice shall
                be included in all copies or substantial portions of the
                Software.

                THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
                KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
                WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
                PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
                OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
                OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
                OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
                SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. */


static char const cb64[] =
		"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";

static const char cd64[] = 
		"|$$$}rstuvwxyz{$$$$$$$>?@ABCDEFGHIJKLMNOPQRSTUVW$$$$$$XYZ[\\]^_`abcdefghijklmnopq";

static void decodeblock(unsigned char in[4], unsigned char out[3])
{   
    out[0] = (unsigned char) (in[0] << 2 | in[1] >> 4);
    out[1] = (unsigned char) (in[1] << 4 | in[2] >> 2);
    out[2] = (unsigned char) (((in[2] << 6) & 0xc0) | in[3]);
}

static void encodeblock(unsigned char in[3], unsigned char out[4], size_t len)
{
    out[0] = cb64[in[0] >> 2];
    out[1] = cb64[((in[0] & 0x03) << 4) | ((in[1] & 0xf0) >> 4)];
    out[2] = (unsigned char) (len > 1 ? cb64[((in[1] & 0x0f) << 2) | ((in[2] & 0xc0) >> 6)] : '=');
    out[3] = (unsigned char) (len > 2 ? cb64[in[2] & 0x3f] : '=');
}

/* End of code by Bob Trower */

/*************************************************************************/
/*************************************************************************/

size_t base64_decode(char const *input, size_t input_len, char *output)
{
	if ((input_len % 4) != 0)
		return 0;
	
	unsigned char in[4], out[3], c;
	size_t result = 0, out_len;
	
	while (input_len)
	{
		out_len = 0;
		
		for (int i = 0; i < 4; ++i)
		{	
			c = *input++;
			
			if (!isalpha(c) && !isdigit(c) && (c != '+') && (c != '/') &&
					(c != '='))
				return 0; /* Invalid character in input! */ 
			
			c = cd64[c - 43];
			c = ((c == '$') ? 0 : (c - 61));
			
			if (c)
			{
				in[i] = (c - 1);
				++out_len;
			} else {
				in[i] = 0;
			}
		}
		
		decodeblock(in, out);
		
		for (size_t i = 0; i < out_len - 1; ++i)
			*output++ = out[i];
		result += (out_len - 1);
		
		input_len -= 4;
	}
	
	return result;
}

/*************************************************************************/

size_t base64_decode_str(char const *input, size_t input_len, char *output)
{
	size_t len = base64_decode(input, input_len, output);
	output[len] = 0;
	return len;
}

/*************************************************************************/

size_t base64_encode(char const *input, size_t input_len, char *output)
{
	unsigned char in[3], out[4];
	size_t result = 0;
	
	while (input_len >= 3)
	{
		in[0] = *input++;
		in[1] = *input++;
		in[2] = *input++;
		
		encodeblock(in, out, 3);
		
		memcpy(output, out, 4);
		output += 4;
		result += 4;
		
		input_len -= 3;
	}
		
	if (input_len)
	{
		in[0] = *input++;
		in[1] = ((input_len == 2) ? *input++ : 0);
		in[2] = 0;
		
		encodeblock(in, out, input_len);
		
		memcpy(output, out, 4);
		result += 4;
	}
	
	return result;
}

/*************************************************************************/

size_t base64_encode_str(char const *input, size_t input_len, char *output)
{
	size_t len = base64_encode(input, input_len, output);
	output[len] = 0;
	return len;
}

/*************************************************************************/

} /* namespace misc */

/*************************************************************************/
