/* Range computation.
 *
 * PegSoft miscellaneous library (c) 2004 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software (the COPYING file); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
 */

#include <system/headers.h>

#include "misc/misc.h"

using namespace std;

/*************************************************************************/

namespace misc
{

/*************************************************************************/

bool complex_range(char const **exprp, unsigned long *first,
		unsigned long *last, bool desc)
{
	if (!exprp || !*exprp)
	{
		*first = 1;
		*last = ULONG_MAX;
		return true;
	}
	
	char const *expr = *exprp;
	
	/* No negative numbers */
	if (*expr == '-')
		return false;
	
	char *end;
	*first = strtoul(expr, &end, 10);
	
	if (!*first)
		return false;
	
	if (*end == ',')
	{
		/* No trailing , with no next range */
		if (!*(++end))
			return false;
	
		*last = *first;
		*exprp = end;
		return true;
	}
			
	if (!*end)
	{
		*last = *first;
		*exprp = end;
		return true;
	}
	
	if (*end++ != '-')
		return false;
	
	/* No negative numbers */
	if (*end == '-')
		return false;
	
	*last = strtoul(end, &end, 10);
	
	if (!*last || (*end && *end != ',') || (!desc && (*last < *first)))
		return false;
	
	/* Trailing , must not be followed by \0 */
	if (*end == ',' && *(++end) == '\0')
		return false;
	
	*exprp = end;
	return true;
}

/*************************************************************************/

bool simple_range(char const *expr, unsigned long *first, unsigned long *last,
		bool desc)
{
	if (!expr)
	{
		*first = 1;
		*last = ULONG_MAX;
		return true;
	}
	
	/* No negative numbers */
	if (*expr == '-')
		return false;
	
	char *end;
	*first = strtoul(expr, &end, 10);
	
	if (!*first)
		return false;
			
	if (!*end)
	{
		*last = *first;
		return true;
	}
	
	if (*end++ != '-')
		return false;
	
	/* No negative numbers */
	if (*end == '-')
		return false;
	
	*last = strtoul(end, &end, 10);
	
	if (!*last || *end || (!desc && (*last < *first)))
		return false;
	
	return true;
}

/*************************************************************************/

} /* namespace misc */

/*************************************************************************/
