/* Monitors base class.
 *
 * PegSoft sockets library (c) 2007 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file COPYING); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <system/headers.h>

#include "sockets/monitor.h"

#if HAVE_POLL
# include "my_poll.h"
#endif
#include "select.h"

using namespace std;

/*************************************************************************/

namespace sockets
{

/*************************************************************************/

monitors_list monitors;
Monitor *default_monitor = 0;

/*************************************************************************/

Monitor::Monitor(std::string const &name, size_t hint) : m_sockets(),
		m_name(name), m_hint(hint)
{
	m_sockets.reserve(m_hint);
}

/*************************************************************************/

bool Monitor::add(Socket *s, int fd, unsigned int events)
{
	if (get_socket(fd))
		return false;
	
	set_socket(fd, s);
	return true;
}

/*************************************************************************/

Socket *Monitor::get_socket(int fd) const
{
	return ((m_sockets.size() > (size_t)fd) ? m_sockets[fd] : 0);
}

/*************************************************************************/

void Monitor::remove(Socket *s, int fd)
{
	set_socket(fd, 0);
}

/*************************************************************************/

void Monitor::set_socket(int fd, Socket *s)
{
	if (fd < 0)
		abort();
	
	if (m_sockets.size() <= (size_t)fd)
		m_sockets.resize((size_t)fd + hint() + 1, 0);
	
	m_sockets[fd] = s;
}

/*************************************************************************/

bool Monitor::update(Socket *s, int fd, unsigned int events)
{
	if (!get_socket(fd))
		return false;
	
	return true;
}

/*************************************************************************/

void do_events()
{
	monitors_list::const_iterator it, end;
		
	for (it = monitors.begin(), end = monitors.end(); it != end; ++it)
		(*it)->do_events();
}

/*************************************************************************/

void setup_monitors(size_t sockcount)
{
	Monitor *select = new Select(sockcount);
	monitors.push_back(select);
	default_monitor = select;
	
#if HAVE_POLL
	Monitor *poll = new Poll(sockcount);
	monitors.push_back(poll);
	default_monitor = poll;
#endif
}

/*************************************************************************/

} /* namespace sockets */

/*************************************************************************/
