/* Command-line input.
 *
 * Epona (c) 2000-2002, 2004 PegSoft
 * Contact us at epona@pegsoft.net
 *
 * Based on the original code of Services by Andy Church.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file COPYING); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <system/headers.h>

#include <iostream>

#include <core/epona.h>
#include <core/version.h>
#include <misc/misc.h>

#include "cmdline.h"
#include "epona.h"
#include "tweak.h"

using namespace std;
using namespace core;

/*************************************************************************/

bool ECmdLineParser::eval()
{	
	ParamsList::const_iterator it, end;
	
	for (it = m_params.begin(), end = m_params.end(); it != end; ++it) 
	{
		if ((it->first == "c") || (it->first == "conf")) {
			if (it->second.empty()) {
				if (it->first == "c")
					m_error = "-c requires a parameter.";
				else
					m_error = "--conf requires a parameter.";
				return false;
			}
			epona->m_conffile = it->second;
		} else if ((it->first == "d") || (it->first == "debug")) {
			epona->debuglevel(epona->debuglevel() + 1);
		} else if (it->first == "debuglevel") {
			if (it->second.empty()) {
				m_error = "--debuglevel requires a parameter.";
				return false;
			}
			epona->debuglevel(atol(it->second.c_str()));
		} else if ((it->first == "D") || (it->first == "define")) {
			/* Just make sure there is a parameter, we'll process
			 * it later. */
			if (it->second.empty()) {
				if (it->first == "D")
					m_error = "-D requires a parameter.";
				else
					m_error = "--define requires a parameter.";
				return false;
			}
		} else if ((it->first == "f") || (it->first == "foreground")) {
			epona->flags_add(Epona::pfForeground);
		} else if ((it->first == "h") || (it->first == "help")) {
			cout << "Usage: " << m_progname << " [options]" << endl;
			cout << "Example: epona -f --readonly" << endl << endl;
			cout << "Options: " << endl << endl;
			cout << "-c, --conf=filename        Set main configuration file "
					"(default: " << DEFAULT_CONF_FILE << ")" << endl;
			cout << "-d, --debug                Increase debug level by 1"
					<< endl;
			cout << "    --debuglevel=level     Set debug level to level"
					<< endl;
			cout << "-D, --define=name[=value]  Set variable for "
					"configuration files" << endl;
			cout << "-f, --foreground           Do not go into background"
					<< endl;
			cout << "-h, --help                 Display this help and exit"
					<< endl;
			cout << "    --iid                  Generate an interface "
					"identifier and exit" << endl; 
			cout << "-I, --include=path         Add another directory for "
					"configuration files path" << endl;
			cout << "-m, --module=name          Load these modules on startup "
					"(ignore the load-module directive)" << endl;
			cout << "-n, --noexpire             Expiration routines don't "
					"run at all" << endl;
			cout << "-q, --quiet                Print only error messages to "
					"the console" << endl;
			cout << "-r, --readonly             Enable read-only mode" << endl;
			cout << "-V, --version              Display version information "
					"and exit" << endl;
			cout << "    --very-quiet           Do not print anything to the "
					"console" << endl;
			cout << endl;
			cout << "There are also various other directives that may be used"
					<< endl;
			cout << "by some modules; read their doc for more information."
					<< endl << endl;
			cout << "Report bugs to <" << PACKAGE_BUGREPORT << ">" << endl; 
			exit(EXIT_SUCCESS);
		} else if (it->first == "iid") {
			/* Generate an interface identifier */
			cout << hex << showbase << time(0) << dec << noshowbase << endl;
			exit(EXIT_SUCCESS);
		} else if ((it->first == "I") || (it->first == "include")) {
			/* Just make sure there is a parameter, we'll process
			 * it later. */
			if (it->second.empty()) {
				if (it->first == "I")
					m_error = "-I requires a parameter.";
				else
					m_error = "--include requires a parameter.";
				return false;
			}
		} else if ((it->first == "m") || (it->first == "module")) {
			/* Just make sure there is a parameter, we'll process
			 * it later. */
			if (it->second.empty()) 
			{
				if (it->first == "m")
					m_error = "-m requires a parameter.";
				else
					m_error = "--module requires a parameter.";
				return false;
			}
		} else if ((it->first == "n") || (it->first == "noexpire")) {
			epona->flags_add(Epona::efNoExpire);
		} else if ((it->first == "q") || (it->first == "quiet")) {
			epona->flags_add(Epona::pfQuiet);
		} else if ((it->first == "r") || (it->first == "readonly")) {
			epona->flags_add(Epona::pfReadOnly);
		} else if ((it->first == "V") || (it->first == "version")) {
			cout << program_name << " " << epona->version_string() << endl;
			exit(EXIT_SUCCESS);
		} else if (it->first == "very-quiet") {
			epona->flags_add(Epona::pfVeryQuiet);
		}
	}
	
	return true;
}

/*************************************************************************/
