/* Lost in memory.
 *
 * Epona (c) 2000-2002, 2004 PegSoft
 * Contact us at epona@pegsoft.net
 *
 * Based on the original code of Services by Andy Church.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file COPYING); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <system/headers.h>

#include <core/epona.h>
#include <log/levels.h>
#include <misc/misc.h>

#include "epona.h"

using namespace std;
using namespace misc;

/*************************************************************************/

LOGLEVELH(WarningMemory, xlog::LogWarning, "Warning::Memory", "warning: memory: ");

/*************************************************************************/

/* Our out-of-memory handler */

void out_of_memory() 
{
	if (epona) {
		epona->fatal("Out of memory!");
	} else {
		abort();
	}
}

/*************************************************************************/

/* smalloc, scalloc, srealloc, sstrdup:
 *	Versions of the memory allocation functions which will cause the
 *	program to terminate with an "Out of memory" error if the memory
 *	cannot be allocated.  (Hence, the return value from these functions
 *	is never NULL.)
 */

/*************************************************************************/

void *smalloc(size_t size)
{
	if (!size) {
		epona->log(WarningMemory(),
				"smalloc: Illegal attempt to allocate 0 bytes");
		size = 1;
	}
	
	void *buf = malloc(size);
	if (!buf)
		out_of_memory();
	
	return buf;
}

/*************************************************************************/

void *scalloc(size_t elsize, size_t els)
{
	if (!elsize || !els) {
		epona->log(WarningMemory(),
				"scalloc: Illegal attempt to allocate 0 bytes");
		elsize = els = 1;
	}
	
	void *buf = calloc(elsize, els);
	if (!buf)
		out_of_memory();
	
	return buf;
}

/*************************************************************************/

void *srealloc(void *oldptr, size_t newsize)
{
	if (!newsize) {
		epona->log(WarningMemory(),
				"srealloc: Illegal attempt to allocate 0 bytes");
		newsize = 1;
	}
	
	void *buf = realloc(oldptr, newsize);
	if (!buf)
		out_of_memory();
	
	return buf;
}

/*************************************************************************/

char *sstrdup(const char *s)
{
	char *t = strdup(s);
	if (!t)
		out_of_memory();
	
	return t;
}

/*************************************************************************/

