/* Bzip2 file module.
 *
 * Bzip2 file module (c) 2005 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file COPYING); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <system/headers.h>

#include <core/version.h>
#include <log/levels.h>
#include <modules/core.h>

#if HAVE_BZLIB
# include "decl.h"
# include "module.h"
#endif

using namespace std;
using namespace modules;
#if HAVE_BZLIB
using namespace bz2file;
#endif

/*************************************************************************/

extern "C" Module *bz2file_get(Core &core)
{
	/* Ensure the core version is the same as the version we were
	 * compiled for so that there are no compatibility problems later. */
	if (core.version() != core::core_version)
	{
		core.log(xlog::LogCritical(), "Core module version mismatch");
		return 0;
	}

#if HAVE_BZLIB
	return (me = new BZ2File(core));
#else
	core.log(xlog::LogError(), "Bzlib was not found on this system");
	return 0;
#endif
}

/*************************************************************************/

extern "C" void bz2file_release(Core &core, Module *module)
{
#if HAVE_BZLIB
	delete module;
	me = 0;
#endif
}

/*************************************************************************/

#if HAVE_BZLIB

namespace bz2file
{

/*************************************************************************/

BZ2File *me;

/*************************************************************************/

BZ2File::BZ2File(Core &core) : Module("bz2file", "Bzip2 file compressor"),
		m_core(&core)
{
	
}

/*************************************************************************/

Interface *BZ2File::factory(PSIID iid)
{
	if (Compressor::IID == iid)
		return new Compressor;
	
	return Module::factory(iid);
}

/*************************************************************************/

} /* namespace bz2file */

#endif /* HAVE_ZLIB */

/*************************************************************************/
