/* Configuration data module.
 *
 * Configuration data module (c) 2006 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file COPYING); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef MODULE_H
#define MODULE_H

/*************************************************************************/

#include <string>

#include <modules/module.h>

#include "records.h"

/*************************************************************************/

namespace conf { class Conf; }
namespace modules { class Core; }

/*************************************************************************/

/// Namespace for module confdata.

namespace confdata 
{

/*************************************************************************/

class Directives;

/*************************************************************************/

/// The module version.
static int const confdata_version = 0x01000000;

/// The confdata module class.

class ConfData : public modules::Module
{
	public:
		/// Constructor.
		ConfData(modules::Core &core);
		/// Destructor.
		~ConfData();
		
		/// Overrides Module::config().
		virtual conf::ConfQ *config(modules::Core &core,
				conf::ConfGlobalQ &global);
		
		/// Overrides Module::load().
		virtual bool load(modules::Core &core);
		
		/// Overrides Module::notify().
		virtual void notify(modules::Module &sender, int msg, void *arg);
		
		/// Overrides Module::query().
		virtual bool query(std::string const &query, std::string &feedback);
		
		/// Overrides Module::start().
		virtual void start(modules::Core &core);
			
		/// Overrides Module::unload().
		virtual void unload(modules::Core &core);
		
		/// Implements Module::version().
		virtual int version() const { return confdata_version; }
		
		/// Returns the core module.
		modules::Core &core() { return *m_core; }
		
		/// Returns the configuration directives.
		Directives const &cd() const { return *m_cd; }
		
		/// Returns the configuration directives collection.
		ConfDirs &confdirs() { return m_confdirs; }
	private:
		modules::Core *m_core;
		Directives *m_cd;
		
		ConfDirs m_confdirs;
		
		bool add_confdirs();
		bool add_confdirs(char const *name, conf::Conf const &parser);
		
		bool remove_confdirs(char const *name);
};

/*************************************************************************/

} /* namespace confdata */

/*************************************************************************/

#endif /* MODULE_H */
