/* Database records and collection classes.
 *
 * Configuration data module (c) 2006 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file COPYING); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef RECORDS_H
#define RECORDS_H

/*************************************************************************/

#include <databases/collection.h>
#include <databases/record.h>
#include <misc/string.h>

/*************************************************************************/

namespace databases { class Database; class Field; }

/*************************************************************************/
	
namespace confdata 
{

/*************************************************************************/

/// Configuration directive record.

class ConfDir : public databases::Record
{
	public:
		/* Data */
		static databases::Field *f_name;
		static databases::Field *f_value;
		
		/* Constants */
		
		/// Maximum length of the mask field.
		static size_t const name_len = 65535;
		/// Maximum length of the creator field.
		static size_t const value_len = 65535 * 8;
	
		/* Functions */
		
		/// Constructor.
		ConfDir();
		/// Constructor that sets the record identifier.
		explicit ConfDir(identifier_type id);
		
		/// Overrides Record::get_s().
		virtual char const *get_s(databases::Field const *field) const;
		
		/// Overrides Record::memory_usage().
		virtual size_t memory_usage() const;
		
		/// Returns directive name.
		char const *name() const { return m_name; }
		/// Sets directive name.
		char const *name(char const *name);
	    
	    /// Returns the directive value. 
		char const *value() const { return m_value; }
		/// Sets the directive value.
		char const *value(char const *value);
	private:
		misc::cstring m_name;
		misc::cstring m_value;
		
		ConfDir(ConfDir const &);
		ConfDir &operator=(ConfDir const &);
};

/*************************************************************************/

/// The configuration directives collection.

class ConfDirs : public databases::Collection
{
	public:
		/// Implements Collection::create().
		virtual databases::Record *
				create(databases::Record::identifier_type id = 0) const;
		
		/// Sets the database up.
		/** Sets the database up. This is called by the Module::notify()
		  * function when the database module is loaded. */
		void setup_database(databases::Database *db);
};

/*************************************************************************/

} /* namespace confdata */

/*************************************************************************/

#endif /* RECORDS_H */
