/* Compressor.
 *
 * Gzip file module (c) 2005 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file COPYING); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <system/headers.h>

#if HAVE_SYS_STAT_H
# include <sys/stat.h>
#endif

#if HAVE_FCNTL_H
# include <fcntl.h>
#endif

#include <misc/string.h>

#include "gzfile.h"

using namespace std;

using misc::strncpyzt;

/*************************************************************************/

static void cleanup(int fd, gzFile zf, char const *path)
{
	if (fd != -1)
		while ((close(fd) == -1) && (errno == EINTR))
			;
	if (zf)
		gzclose(zf);
	remove(path);
}

/*************************************************************************/

namespace gzfile
{

/*************************************************************************/

bool Compressor::compress(char const *path, int level)
{
	*m_error = '\0';
	
	int fd = -1;
	
	if ((fd = open(path, O_RDONLY)) == -1)
	{
		set_errno_error("Read error");
		return false;
	}
	
	char newpath[PATH_MAX];
	snprintf(newpath, sizeof(newpath), "%s.gz", path);
	
	gzFile zf = 0;
	
	if (!(zf = gzopen(newpath, "wb")))
	{
		set_errno_error("Write error");
		cleanup(fd, zf, newpath);
		return false;
	}
	
	if ((level == 0) || (level < 1) || (level > 9))
		level = Z_DEFAULT_COMPRESSION;
	
	if (gzsetparams(zf, level, Z_DEFAULT_STRATEGY) != Z_OK)
	{
		set_zlib_error(zf, "Write error");
		cleanup(fd, zf, newpath);
		return false;
	}
	
	ssize_t bytes = 0;
	char buf[32768];
	
	for (;;)
	{
		while ((bytes = read(fd, buf, sizeof(buf))) == -1)
		{
			if (errno == EINTR)
				continue; 
		
			set_errno_error("Read error");
			cleanup(fd, zf, newpath);
			return false;
		}
		
		if (bytes == 0)
			break;
		
		if (gzwrite(zf, buf, bytes) != bytes)
		{
			set_zlib_error(zf, "Write error");
			cleanup(fd, zf, newpath);
			return false;
		}
	}
	
	if (gzclose(zf) != Z_OK)
	{
		set_zlib_error(zf, "Write error");
		cleanup(fd, zf, newpath);
		return false;
	}
	
	while ((close(fd) == -1) && (errno == EINTR))
		;
	return true;
}

/*************************************************************************/

void Compressor::set_errno_error(char const *prefix)
{
	if (errno)
		errorf("%s: %s", prefix, strerror(errno));
	else
		errorf("%s", prefix);
}

/*************************************************************************/

void Compressor::set_zlib_error(gzFile zf, char const *prefix)
{
	int errnum = 0;
	char const *error = gzerror(zf, &errnum);
	
	if (errnum == Z_ERRNO)
		errorf("%s: %s", prefix, strerror(errno));
	else
		errorf("%s: %s", prefix, error);
}

/*************************************************************************/

} /* namespace gzfile */

/*************************************************************************/
