/* Log file configuration.
 *
 * Log file module (c) 2005 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file COPYING); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef CONF_H
#define CONF_H

/*************************************************************************/

#include <conf/directives.h>
#include <conf/quick.h>
#include <log/conf.h>

/*************************************************************************/

namespace logfile
{

/*************************************************************************/

namespace hidden
{

/*************************************************************************/

class Directives : public conf::DirectivesQ
{
	public:
		/* Classes */
		class Directory : public conf::ConfStrChar
		{
			public:
				Directory();
			protected:
				virtual bool apply(conf::Conf &conf);
		};

		class File : public conf::ConfBlock
		{
			public:
				/* Classes */
				class Rotate : public conf::ConfBoolBlock
				{
					public:
						/* Classes */
						class Suffix : public conf::ConfBoolBlock
						{
							public:
								/* Data */
								conf::ConfStrChar first_part;
								conf::ConfStrChar second_part;
								
								/* Functions */
								/// Constructor.
								Suffix();
								/// Copy constructor.
								Suffix(Suffix const &s);
								/// Assignment operator.
								Suffix &operator=(Suffix const &right);
							protected:
								virtual bool add(conf::Conf &conf,
										conf::Conf &new_conf);
						};

						class Compress : public conf::ConfBoolBlock
						{
							public:
								/* Data */
								conf::ConfString module;
								conf::ConfUint<uint16_t> level;
								
								/* Functions */
								/// Constructor.
								Compress();
								/// Copy constructor.
								Compress(Compress const &c);
								/// Assignment operator.
								Compress &operator=(Compress const &right);
							protected:
								virtual bool add(conf::Conf &conf,
										conf::Conf &new_conf);
						};

						
						/* Data */
						conf::ConfDuration span;
						conf::ConfUint<uint16_t> keep;
						Suffix suffix;
						Compress compress;
						
						/* Functions */
						
						/// Constructor.
						Rotate();
						/// Copy constructor.
						Rotate(Rotate const &r);
						/// Assignment operator
						Rotate &operator=(Rotate const &right);
					protected:
						virtual bool add(conf::Conf &conf, conf::Conf &new_conf);
				};
			
				/* Data */
				conf::ConfStrChar filename;
				xlog::LogRules rules;
				conf::ConfStrChar timestamp_format;
				conf::ConfBool header;
				Rotate rotate;
				
				/* Functions */
				
				/// Constructor.
				File(std::string const &name, bool reloadable);
				/// Copy constructor.
				File(File const &f);
				/// Assignment operator.
				File &operator=(File const &right);
			protected:
				virtual bool add(conf::Conf &conf, conf::Conf &new_conf);
		};
		
		class FileA : public conf::ConfArray<File>
		{
			public:
				FileA();
			protected:
				virtual bool apply_change(conf::Conf &conf);
		};
		
		/* Data */
		Directory directory;
		FileA file;
		
		/* Functions */ 
		Directives(conf::ConfLogger &logger);
		
		bool add(conf::Conf &conf);
	private:
		Directives(Directives const &d);
		Directives &operator=(Directives const &right);
};

/*************************************************************************/

} /* namespace hidden */

/*************************************************************************/

} /* namespace logfile */

/*************************************************************************/

#endif /* CONF_H */
