/* Log file module.
 *
 * Log file module (c) 2005 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file COPYING); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <system/headers.h>

#include <core/version.h>
#include <log/levels.h>
#include <log/logger.h>
#include <modules/core.h>
#include <modules/manager.h>
#include <scheduler/scheduler.h>

#include "coll.h"
#include "conf.h"
#include "decl.h"
#include "module.h"

using namespace std;
using namespace modules;
using namespace logfile;

using xlog::LogLevel;

/*************************************************************************/

extern "C" Module *logfile_get(Core &core)
{
	if (core.version() != core::core_version)
	{
		core.log(xlog::LogCritical(), "Core module version mismatch");
		return 0;
	}
	
	return (me = new LogFile(core));
}

/*************************************************************************/

extern "C" void logfile_release(Core &core, Module *module)
{
	delete module;
	me = 0;
}

/*************************************************************************/

namespace logfile
{

/*************************************************************************/

LogFile *me = 0;

/*************************************************************************/

LogFile::LogFile(Core &core) : Module("logfile", "Log files manager"),
		m_core(&core), m_intf(), 
		m_cd(new hidden::Directives(core.conf_logger())), m_collection(),
		m_tasktype(-1)
{
	
}

/*************************************************************************/

LogFile::~LogFile()
{
	delete m_cd;
}

/*************************************************************************/

conf::ConfQ *LogFile::config(Core &core, conf::ConfGlobalQ &global)
{
	return &m_cd->parser();
}

/*************************************************************************/

bool LogFile::load(Core &core)
{
	/* Load requested modules. */
	hidden::Directives::FileA::container_type::const_iterator it, end;
	
	for (it = m_cd->file->begin(), end = m_cd->file->end(); it != end; ++it)
	{
		if (it->rotate && it->rotate.compress &&
				!core.modules().load(it->rotate.compress.module))
		{
			core.log(LogFileError(), "%s", core.modules().error().c_str());
			if (core.modules().error_code() != ModManager::errUnavailable)
				return false;
		}
	}

	return Module::load(core);
}

/*************************************************************************/

void LogFile::notify(modules::Module &sender, int msg, void *arg)
{
	switch (msg)
	{
		case mnStarted:
		case mnStopped:
			m_collection.module_notify(sender, msg);
			break;
		default:
			break;
	}
	
	Module::notify(sender, msg, arg);
}

/*************************************************************************/

void LogFile::start(Core &core)
{
	/* Register the scheduler task type */
	m_tasktype = core.tasks().register_type();
	
	/* Open the log files */
	hidden::Directives::FileA::container_type::const_iterator it, end;
	
	for (it = m_cd->file->begin(), end = m_cd->file->end(); it != end; ++it)
		if (!m_collection.add(*it))
			core.fatal("Can't start module logfile");
	
	m_collection.cleanup();
	
	/* Register the log interface */
	core.logger().register_interface(m_intf);
	
	/* Call inherited method */
	Module::start(core);
}

/*************************************************************************/

bool LogFile::stop(Core &core)
{
	/* Unregister the log interface */
	core.logger().unregister_interface(m_intf);
	
	/* Close the log files */
	m_collection.clear();
	
	/* Call inherited method */
	return Module::stop(core);
}

/*************************************************************************/

void LogFile::Intf::log(LogLevel const &level, char const *message)
{
	me->collection().log(level, message);
}

/*************************************************************************/

} /* namespace logfile */

/*************************************************************************/
