/* SMTP socket.
 *
 * SMTP mail transport module (c) 2007 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file COPYING); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef SMTP_H
#define SMTP_H

/*************************************************************************/

#include <list>

#include <mail/mail.h>
#include <sockets/sbuffer.h>
#include <sockets/tcpsocket.h>

/*************************************************************************/
	
namespace smtp_cli 
{

/*************************************************************************/

class SMTPSocket : public sockets::TCPSocket
{
	public:
		/// SMTP socket state
		enum
		{
			/// Nothing sent
			msNothing = 0,
			/// HELO has been sent
			msHelo = 1,
			/// MAIL FROM has been sent
			msMailFrom = 2,
			/// RCPT TO has been sent
			msRcptTo = 3,
			/// DATA has been sent
			msData = 4,
			/// Contents has been sent
			msContents = 5
		};
		
		/// Constructor.
		SMTPSocket() : sockets::TCPSocket(&m_myrbuf, &m_mywbuf), m_mail(),
				m_myrbuf(1), m_mywbuf(1), m_state(msNothing) { }
		/// Constructor setting a mail.
		SMTPSocket(mail::Mail const &mail) :
				sockets::TCPSocket(&m_myrbuf, &m_mywbuf), m_mail(mail),
				m_myrbuf(2048), m_mywbuf(8192), m_state(msNothing) { }
		/// Copy constructor.
		SMTPSocket(SMTPSocket const &ss, mail::Mail const &mail) :
				sockets::TCPSocket(ss), m_mail(mail), m_myrbuf(2048),
				m_mywbuf(8192), m_state(msNothing)
				{ rbuf(&m_myrbuf); wbuf(&m_mywbuf); }
		/// Destructor.
		virtual ~SMTPSocket();
		/// Assignment operator -- do not use!
		virtual SMTPSocket &operator=(SMTPSocket const &right);
		
		/// Overrides TCPSocket::write_line().
		virtual ssize_t write_line(char const *line, bool inbuf = false);
	protected:
		/// Overrides TCPSocket::on_connect().
		virtual bool on_connect(int error);
		
		/// Overrides TCPSocket::on_read().
		virtual bool on_read();
		
		/// Overrides TCPSocket::on_write().
		virtual bool on_write();
		
		/// Overrides TCPSocket::on_writing().
		virtual bool on_writing();
	private:
		mail::Mail m_mail;
		
		sockets::SBuffer m_myrbuf;
		sockets::SBuffer m_mywbuf;
		
		uint8_t m_state;
		
		SMTPSocket(SMTPSocket const &ss);
		
		/// Process a line
		bool process(char const *line);
};

/*************************************************************************/

} /* namespace smtp_cli */

/*************************************************************************/

#endif /* SMTP_H */
