/* Twilight database module.
 *
 * Twilight database module (c) 2005 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file COPYING); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef MODULE_H
#define MODULE_H

/*************************************************************************/

#include <databases/xml.h>
#include <modules/module.h>
#include <scheduler/scheduler.h>

/*************************************************************************/

namespace modules { class Core; }

/*************************************************************************/

/// Namespace for module twilight.

namespace twilight 
{

/*************************************************************************/

class Directives;

/*************************************************************************/

/// The module version.
static int const twilight_version = 0x01000000;

/// The twilight module class.

class Twilight : public modules::Module
{
	public:
		/* Classes */
		
		/// The task class.
		class Task : public scheduler::TaskStep
		{
			public:
				/// Constructor.
				Task();
				
				/// Runs the task.
				virtual int run();
				
				/// Overrides scheduler::Task::description().
				virtual std::string description() const
						{ return "Twilight databases automatic saves"; }
				/// Overrides scheduler::Task::name().
				virtual std::string name() const { return "twilight-autosave"; }
		};
	
		/* Functions */
		
		/// Constructor.
		Twilight(modules::Core &core);
		/// Destructor.
		~Twilight();
		
		/// Overrides Module::config().
		virtual conf::ConfQ *config(modules::Core &core,
				conf::ConfGlobalQ &global);
		/// Overrides Module::factory().
		virtual modules::Interface *factory(modules::PSIID iid);
		/// Overrides Module::load().
		virtual bool load(modules::Core &core);
		/// Overrides Module::notify().
		virtual void notify(modules::Module &sender, int msg, void *arg);
		/// Overrides Module::start().
		virtual void start(modules::Core &core);
		/// Overrides Module::stop().
		virtual bool stop(modules::Core &core);
		/// Overrides Module::unload().
		virtual void unload(modules::Core &core);

		/// Saves the index file.
		bool save_index();
		
		/// Implements Module::version().
		virtual int version() const { return twilight_version; }
		
		/// Returns the core module.
		modules::Core &core() { return *m_core; }
		
		/// Returns the configuration directives.
		Directives const &cd() const { return *m_cd; }
		
		/// Returns the database files index.
		databases::XMLIndex &index() { return m_index; }
		
		/// Returns the automatic save task.
		Task &task() { return m_task; }
	private:
		modules::Core *m_core;
		Directives *m_cd;
		
		databases::XMLIndex m_index;
		
		int sched_type;
		Task m_task;
};

/*************************************************************************/

} /* namespace twilight */

/*************************************************************************/

#endif /* MODULE_H */
