/* Simple interfaces to various protocols.
 *
 * Epona (c) 2000-2002, 2004 PegSoft
 * Contact us at epona@pegsoft.net
 *
 * This program is free but copyrighted software; see the file COPYING for
 * details.
 *
 * Based on the original code of Services by Andy Church.
 *
 */

#include <system/headers.h>

#include <list>

#include <core/directives.h>
#include <core/epona.h>
#include <log/levels.h>

#include "epona.h"
#include "netbans.h"
 
#include "services.h"
#include "language.h"
#include "timeout.h"

using namespace std;

/*************************************************************************/

static list<NickChanNetBan *> ql_serv;

/*************************************************************************/

#ifdef HAS_CHGHOST

void s_chghost(char const *nick, char const *host, time_t ts)
{
#if defined(IRC_SEQUANA) || defined(IRC_SEQUANA_LIGHT)
	send_cmd(NULL, "SVSHOST %s :%s", nick, host);
#else
	send_cmd(NULL, "CHGHOST %s :%s", nick, host);
#endif
}

#endif

/*************************************************************************/

/* Negotiates a link to the hub */

void s_linkneg(char const *pass, char const *name, char const *desc)
{
#ifdef IRC_UNREAL
	send_cmd(NULL, "PROTOCTL NICKIP NICKv2 VHP");
#endif
#ifdef IRC_BAHAMUT
    send_cmd(NULL, "PASS %s :TS", pass);
#else
    send_cmd(NULL, "PASS :%s", pass);
#endif
#if defined(IRC_BAHAMUT)
    send_cmd(NULL, "CAPAB NICKIP NOQUIT SSJOIN TS3 TSMODE UNCONNECT "
# if defined(IRC_SEQUANA)
                   "ENCHOST "
#endif
    );
#endif

    send_cmd(NULL, "SERVER %s 1 :%s", name, desc);
#ifdef IRC_BAHAMUT
    send_cmd(NULL, "SVINFO 3 1 0 :%ld", (long) time(NULL));
#endif
}

/*************************************************************************/

/* Sends a channel MODE message */

void s_mode(char const *source, char const *chan, char const *mode, char const *target) 
{
#ifdef IRC_BAHAMUT
	send_cmd(source, "MODE %s 0 %s%s%s", chan, mode, 
			(target ? " " : ""), (target ? target : ""));
#else
	send_cmd(source, "MODE %s %s%s%s", chan, mode, 
			(target ? " " : ""), (target ? target : ""));
#endif
}

/*************************************************************************/

/* Sends a channel MODE message (the mode parameter is prefixed
 * by + automatically) */

void s_mode_plus(char const *source, char const *chan, char mode, char const *target) 
{
#ifdef IRC_BAHAMUT
	send_cmd(source, "MODE %s 0 +%c%s%s", chan, mode, 
			(target ? " " : ""), (target ? target : ""));
#else
	send_cmd(source, "MODE %s +%c%s%s", chan, mode, 
			(target ? " " : ""), (target ? target : ""));
#endif
}

/*************************************************************************/

/* Sends a channel MODE message (the mode parameter is prefixed
 * by - automatically) */

void s_mode_minus(char const *source, char const *chan, char mode, char const *target) 
{
#ifdef IRC_BAHAMUT
	send_cmd(source, "MODE %s 0 -%c%s%s", chan, mode, 
			(target ? " " : ""), (target ? target : ""));
#else
	send_cmd(source, "MODE %s -%c%s%s", chan, mode, 
			(target ? " " : ""), (target ? target : ""));
#endif
}

/*************************************************************************/

/* Introduces a pseudo-client */

void s_nick(char const *nick, char const *user, char const *host,
		char const *real, char const *modes, bool qline)
{
#if defined(IRC_SEQUANA)
	send_cmd(NULL, "NICK %s 1 %ld %s %s %s %s 0 0 %s :%s", nick,
			(long) time(NULL), modes, user, host,
			(char const *)epona->cd().services.sname, host, real);
#elif defined(IRC_BAHAMUT)
    send_cmd(NULL, "NICK %s 1 %ld %s %s %s %s 0 0 :%s", nick,
    		(long) time(NULL), modes, user, host,
    		(char const *)epona->cd().services.sname, real);
#elif defined(IRC_UNREAL)
	send_cmd(NULL, "NICK %s 1 %ld %s %s %s 0 %s * * :%s", nick,
			(long) time(NULL), user, host,
			(char const *)epona->cd().services.sname, modes, real);
#elif defined(IRC_DREAMFORGE)
	send_cmd(NULL, "NICK %s 1 %ld %s %s %s 0 :%s", nick, (long) time(NULL),
			user, host, (char const *)epona->cd().services.sname, real);
	if (strcmp(modes, "+"))
		send_cmd((nick), "MODE %s %s", (nick), (modes));
#endif
	
	if (qline)
	{
		NickChanNetBan *nb = (NickChanNetBan *)netforce.nicknb()->create();
		nb->mask(nick);
		nb->creator(epona->cd().services.sname);
		nb->reason("Reserved for services");
		nb->creation_time(time(0));
		
		if (!netforce.nicknb()->add(nb))
		{
			delete nb;
			return;
		}
		
		nb->apply();
		ql_serv.push_back(nb);
	}
}

/*************************************************************************/

void s_nick_change(char const *nick, char const *newnick, bool qlined)
{
	if (qlined)
	{
		list<NickChanNetBan *>::iterator it, end;
		
		for (it = ql_serv.begin(), end = ql_serv.end(); it != end; ++it)
		{
			if (!stricmp((*it)->mask(), nick))
			{
				if (!netforce.nicknb()->remove(*it))
					break;
				
				ql_serv.erase(it);
				delete (*it);
				
				break;
			}
		}	
	}
	
	send_cmd(nick, "NICK %s", newnick);
	
	if (qlined)
	{
		NickChanNetBan *nb = (NickChanNetBan *)netforce.nicknb()->create();
		nb->mask(newnick);
		nb->creator(epona->cd().services.sname);
		nb->reason("Reserved for services");
		nb->creation_time(time(0));
		
		if (!netforce.nicknb()->add(nb))
		{
			delete nb;
			return;
		}
		
		nb->apply();
		ql_serv.push_back(nb);
	}
}

/*************************************************************************/

void s_quit(char const *nick, char const *reason, bool qlined)
{
	send_cmd(nick, "QUIT :Quit: %s", reason);
	
	if (qlined)
	{
		list<NickChanNetBan *>::iterator it, end;
		
		for (it = ql_serv.begin(), end = ql_serv.end(); it != end; ++it)
		{
			if (!stricmp((*it)->mask(), nick))
			{
				if (!netforce.nicknb()->remove(*it))
					break;
				
				ql_serv.erase(it);
				delete (*it);
				
				break;
			}
		}	
	}
}

/*************************************************************************/

void s_squit(char const *server, char const *reason)
{
	send_cmd(epona->cd().services.sname, "SQUIT %s :%s", server, reason);
}

/*************************************************************************/

void s_svsmode_user(char const *source, char const *nick, time_t timestamp, char const *modes, char const *arg)
{
#if defined(IRC_BAHAMUT)
	send_cmd(epona->cd().services.sname, "SVSMODE %s %ld %s%s%s", nick,
			(long) timestamp, modes, (arg ? " " : ""), (arg ? arg : ""));
#elif defined(IRC_UNREAL)
	send_cmd(source, "SVS2MODE %s %s%s%s", nick, modes, (arg ? " " : ""),
			(arg ? arg : ""));
#else
	send_cmd(epona->cd().services.sname, "SVSMODE %s %s%s%s", nick, modes,
			(arg ? " " : ""), (arg ? arg : ""));
#endif
}

/*************************************************************************/

void s_svsnick(char const *service, User *u)
{
	static int counter = time(0) % 0xFFFFF;
	
	char newnick[NICKMAX];
	snprintf(newnick, sizeof(newnick), "%s%X",
			epona->cd().services.guest_nick.value().c_str(), counter);
	
	notice_lang(service, u, FORCENICKCHANGE_CHANGING, newnick);
	send_cmd(NULL, "SVSNICK %s %s :%ld", u->nick, newnick, time(0));
	
	if (++counter > 0xFFFFF)
		counter = 0;
}

/*************************************************************************/

void s_svsnoop(char *server, int set)
{
	send_cmd(NULL, "SVSNOOP %s %s", server, (set ? "+" : "-"));	
}

/*************************************************************************/
