/* Signal hunting.
 *
 * Epona (c) 2000-2002, 2004 PegSoft
 * Contact us at epona@pegsoft.net
 *
 * Based on the original code of Services by Andy Church.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file COPYING); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <system/headers.h>

#include <csignal>

#include <core/directives.h>
#include <core/epona.h>
#include <log/levels.h>
#include <scheduler/scheduler.h>

#include "epona.h"

#include "services.h"
#include "extern.h"

using namespace std;
using namespace core;

/*************************************************************************/

LOGLEVEL (CriticalProgram, xlog::LogCritical, "Critical::Program");
LOGLEVELH(CriticalProgramPanic, CriticalProgram,
		"Critical::Program::Panic", "");

/*************************************************************************/

static RETSIGTYPE fatal_signal(int signum)
{
	if (epona->started()) 
	{
		if (!waiting) 
		{
			epona->log(CriticalProgramPanic(), "PANIC! buffer = %s", inbuf);
			/* Cut off if this would make IRC command >510 characters. */
			if (strlen(inbuf) > 448) {
				inbuf[446] = '>';
				inbuf[447] = '>';
				inbuf[448] = 0;
			}
			wallops(NULL, "PANIC! buffer = %s\r\n", inbuf);
		} else if (waiting < 0) {
			/* This is static on the off-chance we run low on stack */
			static char buf[BUFSIZE];
	    	
			switch (waiting) 
			{
				case  -1:
					if (epona->tasks().current_task())
						snprintf(buf, sizeof(buf), "in scheduler (%s)", 
								epona->tasks().current_task()->description().c_str());
					else
						snprintf(buf, sizeof(buf), "in scheduler");
					break;
				case  -2:
					snprintf(buf, sizeof(buf), "in timed_update");
					break;
				case  -3:
					snprintf(buf, sizeof(buf), "in sockets::do_events");
					break;
				case -18:
					snprintf(buf, sizeof(buf), "blacklist update");
					break;
				case -21:
					snprintf(buf, sizeof(buf), "expiring nicknames");
					break;
				case -22:
					snprintf(buf, sizeof(buf), "expiring channels");
					break;
				case -25:
					snprintf(buf, sizeof(buf), "expiring autokills");
					break;
				case -26:
					snprintf(buf, sizeof(buf), "expiring SGLINEs");
					break;
				case -27:
					snprintf(buf, sizeof(buf), "expiring SZLINEs");
					break;
				case -28:
					snprintf(buf, sizeof(buf), "expiring SQLINEs");
					break;
				default : 	
					snprintf(buf, sizeof(buf), "waiting=%d", waiting);
			}
			wallops(NULL, "PANIC! %s (%s)", buf, strsignal(signum));
			epona->log(CriticalProgramPanic(), "PANIC! %s (%s)", buf,
					strsignal(signum));
		}
	}
	
	epona->fatal(strsignal(signum));
}

/*************************************************************************/

static RETSIGTYPE interruption_signal(int signum)
{
	epona->fatalf("Got signal: %s", strsignal(signum));
}

/*************************************************************************/

static RETSIGTYPE restart_signal(int signum)
{
	snprintf(epona->exitmsg, epona->exitmsg_size, "Got signal: %s",
			strsignal(signum));
	epona->schedule(Epona::sfRestart | Epona::sfSync);
}

/*************************************************************************/

static RETSIGTYPE stop_signal(int signum)
{
	snprintf(epona->exitmsg, epona->exitmsg_size, "Got signal: %s",
			strsignal(signum));
	epona->schedule(Epona::sfStop | Epona::sfSync);
}

/*************************************************************************/

void Epona::dump_core(bool dump)
{
	signal(SIGBUS,  (dump ? SIG_DFL : fatal_signal));
	signal(SIGFPE,  (dump ? SIG_DFL : fatal_signal));
	signal(SIGILL,  (dump ? SIG_DFL : fatal_signal));
	signal(SIGIOT,  (dump ? SIG_DFL : fatal_signal));
	signal(SIGSEGV, (dump ? SIG_DFL : fatal_signal));
	signal(SIGTRAP, (dump ? SIG_DFL : fatal_signal));
}

/*************************************************************************/

void Epona::init_signals()
{
#ifdef NSIG
	for (int i = 1; i < NSIG; i++) 
	{
#else
	for (int i = 1; i < 32; i++) 
	{
#endif
		/* SIGKILL and SIGSTOP can't be caught. */
		if (i == SIGKILL || i == SIGSTOP)
			continue;

		signal(i, SIG_IGN);
	}
	
	signal(SIGHUP,  restart_signal);
	signal(SIGTERM, stop_signal);
	
	signal(SIGINT,  interruption_signal);
	signal(SIGQUIT, interruption_signal);
	
	dump_core(m_cd->dump_core);
}

/*************************************************************************/
